import {
  TIME_BEFORE_EXPIRING_JWT_IN_SECONDS
} from "./chunk-BMYZMMVR.js";
import {
  deleteCookie,
  getCookie,
  setCookie
} from "./chunk-5JXPQKM4.js";
import {
  getJWTExpiration
} from "./chunk-MLMLBCSI.js";
import {
  __async,
  __spreadProps,
  __spreadValues
} from "./chunk-JPRRZPBL.js";

// src/CrossmintAuthClient.ts
import {
  AUTH_SDK_ROOT_ENDPOINT,
  CROSSMINT_API_VERSION,
  CrossmintAuth,
  CrossmintAuthenticationError,
  REFRESH_TOKEN_PREFIX,
  SESSION_PREFIX
} from "@crossmint/common-sdk-auth";
import { queueTask } from "@crossmint/client-sdk-base";
var CrossmintAuthClient = class _CrossmintAuthClient extends CrossmintAuth {
  constructor(crossmint, apiClient, config = {}) {
    var _a, _b;
    super(crossmint, apiClient, config);
    this.refreshTask = null;
    this.refreshPromise = null;
    this.callbacks = (_a = config.callbacks) != null ? _a : {};
    this.logoutRoute = (_b = config.logoutRoute) != null ? _b : null;
  }
  static from(crossmint, config = {}) {
    const authClient = new _CrossmintAuthClient(crossmint, CrossmintAuth.defaultApiClient(crossmint), config);
    if (typeof window !== "undefined") {
      authClient.handleRefreshAuthMaterial();
    }
    return authClient;
  }
  getUser() {
    return __async(this, null, function* () {
      try {
        const response = yield this.apiClient.get(`api/${CROSSMINT_API_VERSION}/sdk/auth/user`, {
          headers: { "Content-Type": "application/json" }
        });
        if (!response.ok) {
          throw yield response.text();
        }
        return yield response.json();
      } catch (error) {
        throw new CrossmintAuthenticationError(
          `Failed to fetch user: ${error instanceof Error ? error.message : "Unknown error"}`
        );
      }
    });
  }
  storeAuthMaterial(authMaterial) {
    setCookie(SESSION_PREFIX, authMaterial.jwt);
    setCookie(REFRESH_TOKEN_PREFIX, authMaterial.refreshToken.secret, authMaterial.refreshToken.expiresAt);
  }
  logout() {
    return __async(this, null, function* () {
      var _a, _b;
      const oldRefreshToken = getCookie(REFRESH_TOKEN_PREFIX);
      deleteCookie(REFRESH_TOKEN_PREFIX);
      deleteCookie(SESSION_PREFIX);
      (_b = (_a = this.callbacks).onLogout) == null ? void 0 : _b.call(_a);
      try {
        if (this.logoutRoute != null) {
          yield this.logoutFromCustomRoute();
        } else if (oldRefreshToken != null) {
          yield this.logoutFromDefaultRoute(oldRefreshToken);
        }
      } catch (error) {
        console.error(error);
      }
    });
  }
  handleRefreshAuthMaterial(refreshTokenSecret) {
    return __async(this, null, function* () {
      var _a, _b;
      const refreshToken = refreshTokenSecret != null ? refreshTokenSecret : getCookie(REFRESH_TOKEN_PREFIX);
      if (refreshToken == null && this.refreshRoute == null) {
        return;
      }
      try {
        if (this.refreshPromise == null) {
          this.refreshPromise = this.refreshAuthMaterial(refreshToken);
        }
        const authMaterial = yield this.refreshPromise;
        if (this.refreshRoute == null) {
          this.storeAuthMaterial(authMaterial);
        }
        (_b = (_a = this.callbacks).onTokenRefresh) == null ? void 0 : _b.call(_a, authMaterial);
        this.scheduleNextRefresh(authMaterial.jwt);
      } catch (error) {
        console.error(error);
        this.logout();
      } finally {
        this.refreshPromise = null;
      }
    });
  }
  getOAuthUrl(provider) {
    return __async(this, null, function* () {
      try {
        const response = yield this.apiClient.get(`${AUTH_SDK_ROOT_ENDPOINT}/social/${provider}/start`, {
          headers: { "Content-Type": "application/json" }
        });
        if (!response.ok) {
          throw yield response.text();
        }
        const data = yield response.json();
        return data.oauthUrl;
      } catch (error) {
        throw new CrossmintAuthenticationError(
          `Failed to get OAuth URL for provider ${provider}: ${error instanceof Error ? error.message : "Unknown error"}`
        );
      }
    });
  }
  sendEmailOtp(email) {
    return __async(this, null, function* () {
      try {
        const response = yield this.apiClient.post(`${AUTH_SDK_ROOT_ENDPOINT}/otps/send`, {
          headers: { "Content-Type": "application/json" },
          body: JSON.stringify({ email })
        });
        if (!response.ok) {
          throw yield response.text();
        }
        return yield response.json();
      } catch (error) {
        throw new CrossmintAuthenticationError(
          `Failed to send email OTP: ${error instanceof Error ? error.message : "Unknown error"}`
        );
      }
    });
  }
  confirmEmailOtp(email, emailId, token) {
    return __async(this, null, function* () {
      try {
        const queryParams = new URLSearchParams({
          email,
          signinAuthenticationMethod: "email",
          token,
          locale: "en",
          state: emailId
        });
        const response = yield this.apiClient.post(`${AUTH_SDK_ROOT_ENDPOINT}/authenticate?${queryParams}`, {
          headers: { "Content-Type": "application/json" }
        });
        if (!response.ok) {
          throw yield response.text();
        }
        const resData = yield response.json();
        return resData.oneTimeSecret;
      } catch (error) {
        throw new CrossmintAuthenticationError(
          `Failed to confirm email OTP: ${error instanceof Error ? error.message : "Unknown error"}`
        );
      }
    });
  }
  signInWithFarcaster(data) {
    return __async(this, null, function* () {
      try {
        const queryParams = new URLSearchParams({
          signinAuthenticationMethod: "farcaster",
          callbackUrl: `${this.apiClient.baseUrl}/${AUTH_SDK_ROOT_ENDPOINT}/callback`
        });
        const response = yield this.apiClient.post(`${AUTH_SDK_ROOT_ENDPOINT}/authenticate?${queryParams}`, {
          headers: { "Content-Type": "application/json" },
          body: JSON.stringify(__spreadProps(__spreadValues({}, data), {
            domain: data.signatureParams.domain,
            redirect: true,
            callbackUrl: `${this.apiClient.baseUrl}/${AUTH_SDK_ROOT_ENDPOINT}/callback`
          }))
        });
        if (!response.ok) {
          throw yield response.text();
        }
        const resData = yield response.json();
        return resData.oneTimeSecret;
      } catch (error) {
        throw new CrossmintAuthenticationError(
          `Failed to sign in with Farcaster: ${error instanceof Error ? error.message : "Unknown error"}`
        );
      }
    });
  }
  signInWithSmartWallet(address) {
    return __async(this, null, function* () {
      try {
        const queryParams = new URLSearchParams({ signinAuthenticationMethod: "evm" });
        const response = yield this.apiClient.post(
          `${AUTH_SDK_ROOT_ENDPOINT}/crypto_wallets/authenticate/start?${queryParams}`,
          {
            headers: { "Content-Type": "application/json" },
            body: JSON.stringify({ walletAddress: address })
          }
        );
        if (!response.ok) {
          throw yield response.text();
        }
        return yield response.json();
      } catch (error) {
        throw new CrossmintAuthenticationError(
          `Failed to initiate smart wallet sign in: ${error instanceof Error ? error.message : "Unknown error"}`
        );
      }
    });
  }
  authenticateSmartWallet(address, signature) {
    return __async(this, null, function* () {
      try {
        const queryParams = new URLSearchParams({
          signinAuthenticationMethod: "evm",
          callbackUrl: `${this.apiClient.baseUrl}/${AUTH_SDK_ROOT_ENDPOINT}/callback`
        });
        const response = yield this.apiClient.post(
          `${AUTH_SDK_ROOT_ENDPOINT}/crypto_wallets/authenticate?${queryParams}`,
          {
            headers: { "Content-Type": "application/json" },
            body: JSON.stringify({ walletAddress: address, signature })
          }
        );
        if (!response.ok) {
          throw yield response.text();
        }
        return yield response.json();
      } catch (error) {
        throw new CrossmintAuthenticationError(
          `Failed to authenticate smart wallet: ${error instanceof Error ? error.message : "Unknown error"}`
        );
      }
    });
  }
  logoutFromCustomRoute() {
    return __async(this, null, function* () {
      if (!this.logoutRoute) {
        throw new Error("Custom logout route is not set");
      }
      return yield fetch(this.logoutRoute, { method: "POST" });
    });
  }
  scheduleNextRefresh(jwt) {
    const jwtExpiration = getJWTExpiration(jwt);
    if (!jwtExpiration) {
      throw new Error("Invalid JWT");
    }
    const currentTime = Date.now() / 1e3;
    const timeToExpire = jwtExpiration - currentTime - TIME_BEFORE_EXPIRING_JWT_IN_SECONDS;
    if (timeToExpire > 0) {
      const endTime = Date.now() + timeToExpire * 1e3;
      this.cancelScheduledRefresh();
      this.refreshTask = queueTask(() => this.handleRefreshAuthMaterial(), endTime);
    }
  }
  cancelScheduledRefresh() {
    if (this.refreshTask) {
      this.refreshTask.cancel();
      this.refreshTask = null;
    }
  }
};

export {
  CrossmintAuthClient
};
//# sourceMappingURL=chunk-SGSYRRRS.js.map